@@ -15,98 +15,113 @@ the password is valid for the user.
 */
 
 /*
- * $Id: auth-passwd.c,v 1.19 1998/07/08 01:44:46 kivinen Exp $
+ * $Id: auth-passwd.c,v 1.23 1999/04/29 07:52:02 tri Exp $
  * $Log: auth-passwd.c,v $
+ * Revision 1.23  1999/04/29 07:52:02  tri
+ * 	Replaced OSF1/C2 security support with more complete SIA
+ *         (Security Integration Architecture) support by Tom Woodburn.
+ *
+ * Revision 1.22  1999/02/23 07:06:14  tri
+ *      Fixed compilation errors.
+ *
+ * Revision 1.21  1999/02/22 08:13:56  tri
+ *      Final fixes for 1.2.27.
+ *
+ * Revision 1.20  1999/02/21 19:51:52  ylo
+ *      Intermediate commit of ssh1.2.27 stuff.
+ *      Main change is sprintf -> snprintf; however, there are also
+ *      many other changes.
+ *
  * Revision 1.19  1998/07/08 01:44:46  kivinen
- * 	Added one missing space.
+ *      Added one missing space.
  *
  * Revision 1.18  1998/07/08 00:36:44  kivinen
- * 	Changed to use PASSWD_PATH. Better HPUX TCB AUTH support.
+ *      Changed to use PASSWD_PATH. Better HPUX TCB AUTH support.
  *
  * Revision 1.17  1998/06/11 00:03:38  kivinen
- * 	Added username to /bin/password commands.
+ *      Added username to /bin/password commands.
  *
  * Revision 1.16  1998/05/23  20:19:40  kivinen
- * 	Removed #define uint32 rpc_unt32, because md5 uint32 is now
- * 	md5_uint32. Changed () -> (void). Changed osf1c2_getprpwent
- * 	function to return true/false. Added
- * 	forced_empty_passwd_change support.
+ *      Removed #define uint32 rpc_unt32, because md5 uint32 is now
+ *      md5_uint32. Changed () -> (void). Changed osf1c2_getprpwent
+ *      function to return true/false. Added
+ *      forced_empty_passwd_change support.
  *
  * Revision 1.15  1998/05/11  21:27:47  kivinen
- * 	Set correct_passwd to contain 255...255 so even if some
- * 	function doesn't set it, it cannot contain empty password.
+ *      Set correct_passwd to contain 255...255 so even if some
+ *      function doesn't set it, it cannot contain empty password.
  *
  * Revision 1.14  1998/04/30 01:50:20  kivinen
- * 	Added code that will force /bin/passwd command if password is
- * 	expired.
+ *      Added code that will force /bin/passwd command if password is
+ *      expired.
  *
  * Revision 1.13  1998/03/27 16:53:09  kivinen
- * 	Added aix authenticate function support.
+ *      Added aix authenticate function support.
  *
  * Revision 1.12  1998/01/02 06:14:31  kivinen
- * 	Fixed kerberos ticket name handling. Added OSF C2 account
- * 	locking and expiration support.
+ *      Fixed kerberos ticket name handling. Added OSF C2 account
+ *      locking and expiration support.
  *
  * Revision 1.11  1997/04/17 03:57:05  kivinen
- * 	Kept FILE: prefix in kerberos ticket filename as DCE cache
- * 	code requires it (patch from Doug Engert <DEEngert@anl.gov>).
+ *      Kept FILE: prefix in kerberos ticket filename as DCE cache
+ *      code requires it (patch from Doug Engert <DEEngert@anl.gov>).
  *
  * Revision 1.10  1997/04/05 21:45:25  kivinen
- * 	Changed verify_krb_v5_tgt to take *error_code instead of
- * 	error_code.
+ *      Changed verify_krb_v5_tgt to take *error_code instead of
+ *      error_code.
  *
  * Revision 1.9  1997/03/27 03:09:20  kivinen
- * 	Added kerberos patches from Glenn Machin.
+ *      Added kerberos patches from Glenn Machin.
  *
  * Revision 1.8  1997/03/26 06:59:18  kivinen
- * 	Changed uid 0 to UID_ROOT.
+ *      Changed uid 0 to UID_ROOT.
  *
  * Revision 1.7  1997/03/19 15:57:21  kivinen
- * 	Added SECURE_RPC, SECURE_NFS and NIS_PLUS support from Andy
- * 	Polyakov <appro@fy.chalmers.se>.
+ *      Added SECURE_RPC, SECURE_NFS and NIS_PLUS support from Andy
+ *      Polyakov <appro@fy.chalmers.se>.
  *
  * Revision 1.6  1996/10/30 04:22:43  kivinen
- * 	Added #ifdef HAVE_SHADOW_H around shadow.h including.
+ *      Added #ifdef HAVE_SHADOW_H around shadow.h including.
  *
  * Revision 1.5  1996/10/29 22:33:59  kivinen
- * 	log -> log_msg.
+ *      log -> log_msg.
  *
  * Revision 1.4  1996/10/08 13:50:44  ttsalo
- * 	Allow long passwords for HP-UX TCB authentication
+ *      Allow long passwords for HP-UX TCB authentication
  *
  * Revision 1.3  1996/09/08 17:36:51  ttsalo
- * 	Patches for HPUX 10.x shadow passwords from
- * 	vincent@ucthpx.uct.ac.za (Russell Vincent) merged.
+ *      Patches for HPUX 10.x shadow passwords from
+ *      vincent@ucthpx.uct.ac.za (Russell Vincent) merged.
  *
  * Revision 1.2  1996/02/18 21:53:45  ylo
- * 	Test for HAVE_ULTRIX_SHADOW_PASSWORDS instead of ultrix
- * 	(mips-dec-mach3 has ultrix defined, but does not support the
- * 	shadow password stuff).
+ *      Test for HAVE_ULTRIX_SHADOW_PASSWORDS instead of ultrix
+ *      (mips-dec-mach3 has ultrix defined, but does not support the
+ *      shadow password stuff).
  *
  * Revision 1.1.1.1  1996/02/18 21:38:12  ylo
- * 	Imported ssh-1.2.13.
+ *      Imported ssh-1.2.13.
  *
  * Revision 1.8  1995/09/27  02:10:34  ylo
- * 	Added support for SCO unix shadow passwords.
+ *      Added support for SCO unix shadow passwords.
  *
  * Revision 1.7  1995/09/10  22:44:41  ylo
- * 	Added OSF/1 C2 extended security stuff.
+ *      Added OSF/1 C2 extended security stuff.
  *
  * Revision 1.6  1995/08/21  23:20:29  ylo
- * 	Fixed a typo.
+ *      Fixed a typo.
  *
  * Revision 1.5  1995/08/19  13:15:56  ylo
- * 	Changed securid code to initialize itself only once.
+ *      Changed securid code to initialize itself only once.
  *
  * Revision 1.4  1995/08/18  22:42:51  ylo
- * 	Added General Dynamics SecurID support from Donald McKillican
- * 	<dmckilli@qc.bell.ca>.
+ *      Added General Dynamics SecurID support from Donald McKillican
+ *      <dmckilli@qc.bell.ca>.
  *
  * Revision 1.3  1995/07/13  01:12:34  ylo
- * 	Removed the "Last modified" header.
+ *      Removed the "Last modified" header.
  *
  * Revision 1.2  1995/07/13  01:09:50  ylo
- * 	Added cvs log.
+ *      Added cvs log.
  *
  * $Endlog$
  */
@@ -143,6 +158,10 @@ the password is valid for the user.
 #include "servconf.h"
 #include "xmalloc.h"
 
+#ifdef HAVE_SIA
+#include "sshsia.h"
+#endif /* HAVE_SIA */
+
 #ifdef SECURE_RPC
 /*
  * refer to ftp://playground.sun.com/pub/rpc/tirpcsrc2.3.tar.Z for
@@ -184,8 +203,8 @@ void keylogout(void)
   /* revoke secret key from keyserv(1m) */
   memset(secret, '\0', sizeof(secret));
   key_setsecret(secret);     /* this one is keyserv.1 interface,
-				but it does the job even for keyserv.2
-				and takes the only argument:-) */
+                                but it does the job even for keyserv.2
+                                and takes the only argument:-) */
 #ifdef SECURE_NFS
   /* as well as from nfs client module */
   nfs_revauth(uid_keylogged);
@@ -236,33 +255,33 @@ int keylogin(const char *passwd)
   char secret[HEXKEYBYTES], public[HEXKEYBYTES];
 #endif /* KEY_VERS2 */
   
-  if (getnetname(netnam) &&	/* do i exists?            */
+  if (getnetname(netnam) &&     /* do i exists?            */
       !(ret = my_secretkey_is_set(netnam)) && /* is it already set?      */
-      passwd)			/* do i have the password? */
+      passwd)                   /* do i have the password? */
     {
       strncpy(phrase, passwd, 8);
       phrase[8] = '\0';
       memset(public, '\0', sizeof(public));
       memset(secret, '\0', sizeof(secret));
       if (getsecretkey(netnam, secret, phrase) && *secret)
-	{
+        {
 #ifdef KEY_VERS2
-	  /*
-	   * key_setnet is keyserv.2 interface and is not documented,
-	   * but used by keylogin(1). and it's real mess with return
-	   * values:-( so far, namely up to Solaris 2.5.1, it returns
-	   * 1 on success and -1 when fails.
-	   */
-	  if (ret = (key_setnet(&key_setnet_arg) > 0))
+          /*
+           * key_setnet is keyserv.2 interface and is not documented,
+           * but used by keylogin(1). and it's real mess with return
+           * values:-( so far, namely up to Solaris 2.5.1, it returns
+           * 1 on success and -1 when fails.
+           */
+          if (ret = (key_setnet(&key_setnet_arg) > 0))
 #else /* KEY_VERS2 */
-	  if (ret = !key_setsecret(secret))
+          if (ret = !key_setsecret(secret))
 #endif /* KEY_VERS2 */
-	    {
-	      uid_keylogged = geteuid(); /* save it for keylogout */
-	      atexit(keylogout_atexit); /* clean up after ourselves */
-	    }
-	  memset(secret, '\0', sizeof (secret));
-	}
+            {
+              uid_keylogged = geteuid(); /* save it for keylogout */
+              atexit(keylogout_atexit); /* clean up after ourselves */
+            }
+          memset(secret, '\0', sizeof (secret));
+        }
       memset(phrase, '\0', sizeof(phrase));
     }
   return ret; /* secret key was set by whomever */
@@ -318,7 +337,7 @@ extern char *ticket;
  */
 static
 int verify_krb_v5_tgt (krb5_context c, krb5_ccache ccache,
-		       krb5_error_code *error_code)
+                       krb5_error_code *error_code)
 {
   char phost[BUFSIZ];
   int retval, have_keys;
@@ -349,7 +368,7 @@ int verify_krb_v5_tgt (krb5_context c, krb5_ccache ccache,
   /* (use default keytab, kvno IGNORE_VNO to get the first match,
      and enctype is currently ignored anyhow.) */
   krbval = krb5_kt_read_service_key (c, NULL, princ, 0,
-				     ENCTYPE_DES_CBC_CRC, &kb);
+                                     ENCTYPE_DES_CBC_CRC, &kb);
   if (kb)
     krb5_free_keyblock (c, kb);
   /* any failure means we don't have keys at all. */
@@ -357,7 +376,7 @@ int verify_krb_v5_tgt (krb5_context c, krb5_ccache ccache,
   
   /* talk to the kdc and construct the ticket */
   krbval = krb5_mk_req(c, &auth_context, 0, "host", phost,
-		       0, ccache, &packet);
+                       0, ccache, &packet);
   /* wipe the auth context for mk_req */
   if (auth_context)
     {
@@ -367,54 +386,54 @@ int verify_krb_v5_tgt (krb5_context c, krb5_ccache ccache,
   if (krbval == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN)
     {
       /* we have a service key, so something should be 
-	 in the database, therefore this error packet could
-	 have come from an attacker. */
+         in the database, therefore this error packet could
+         have come from an attacker. */
       if (have_keys)
-	{
-	  retval = -1;
-	  goto EGRESS;
-	}
+        {
+          retval = -1;
+          goto EGRESS;
+        }
       /* but if it is unknown and we've got no key, we don't
-	 have any security anyhow, so it is ok. */
+         have any security anyhow, so it is ok. */
       else
-	{
-	  retval = 0;
-	  goto EGRESS;
-	}
+        {
+          retval = 0;
+          goto EGRESS;
+        }
     }
   else
     if (krbval)
       {
-	*error_code = krbval;
-	retval = -1;
-	goto EGRESS;
+        *error_code = krbval;
+        retval = -1;
+        goto EGRESS;
       }
   /* got ticket, try to use it */
   krbval = krb5_rd_req(c, &auth_context, &packet, 
-		       princ, NULL, NULL, &ticket);
+                       princ, NULL, NULL, &ticket);
   
   
   if (krbval)
     {
       if (!have_keys)
-	/* The krb5 errors aren't specified well, but I think
-	   these values cover the cases we expect.  */
-	switch (krbval)
-	  {
-	    /* no keytab */
-	  case KRB5_KT_NOTFOUND:
-	  case ENOENT:
-	    /* keytab found, missing entry */
-	    retval = 0;
-	    break;
-	  default:
-	    /* unexpected error: fail */
-	    retval = -1;
-	    break;
-	  }
+        /* The krb5 errors aren't specified well, but I think
+           these values cover the cases we expect.  */
+        switch (krbval)
+          {
+            /* no keytab */
+          case KRB5_KT_NOTFOUND:
+          case ENOENT:
+            /* keytab found, missing entry */
+            retval = 0;
+            break;
+          default:
+            /* unexpected error: fail */
+            retval = -1;
+            break;
+          }
       else
-	/* Any error here is bad.  */
-	retval = -1;
+        /* Any error here is bad.  */
+        retval = -1;
       *error_code = krbval;
       goto EGRESS;
     }
@@ -466,7 +485,7 @@ krb5_preauthtype * preauth = preauth_list;
    authentication succeeds. */
 #ifdef KERBEROS
 int auth_password(const char *server_user, const char *password,
-		  krb5_principal client)
+                  krb5_principal client)
 #else  /* KERBEROS */
 int auth_password(const char *server_user, const char *password)
 #endif /* KERBEROS */
@@ -522,33 +541,33 @@ int auth_password(const char *server_user, const char *password)
   if (options.kerberos_authentication)
     {
 #if defined(KRB5)
-      sprintf(ccname, "FILE:/tmp/krb5cc_l%d", getpid());
+      snprintf(ccname, sizeof(ccname), "FILE:/tmp/krb5cc_l%d", getpid());
       
       if (problem = krb5_cc_resolve(ssh_context, ccname, &ccache))
-	goto errout2;
+        goto errout2;
       
       if (problem = krb5_cc_initialize(ssh_context, ccache, client))
-	goto errout;
+        goto errout;
       
       problem =
-	krb5_build_principal_ext(ssh_context, &server,
-				 krb5_princ_realm(ssh_context, client)->length,
-				 krb5_princ_realm(ssh_context, client)->data,
-				 tgtname.length, tgtname.data,
-				 krb5_princ_realm(ssh_context, client)->length,
-				 krb5_princ_realm(ssh_context, client)->data,
-				 0);
+        krb5_build_principal_ext(ssh_context, &server,
+                                 krb5_princ_realm(ssh_context, client)->length,
+                                 krb5_princ_realm(ssh_context, client)->data,
+                                 tgtname.length, tgtname.data,
+                                 krb5_princ_realm(ssh_context, client)->length,
+                                 krb5_princ_realm(ssh_context, client)->data,
+                                 0);
       if (problem)
-	goto errout;
+        goto errout;
       
       memset(&my_creds, 0, sizeof(my_creds));
       my_creds.client = client;
       my_creds.server = server;
       problem = krb5_timeofday(ssh_context, &now);
       if (problem)
-	goto errout;
+        goto errout;
       my_creds.times.starttime = 0; /* start timer when
-				       request gets to KDC */
+                                       request gets to KDC */
       my_creds.times.endtime = now + 60*60*10;   /* 10 hours */
       
       problem = krb5_string_to_deltat("7d", &rlife); /* 7 days renew time */
@@ -559,85 +578,86 @@ int auth_password(const char *server_user, const char *password)
       
       
       problem = krb5_get_in_tkt_with_password(ssh_context, krb5_options, 0,
-					      NULL, preauth,
-					      password, ccache,
-					      &my_creds, 0);
+                                              NULL, preauth,
+                                              password, ccache,
+                                              &my_creds, 0);
       
       /* If not successful try no preauthentication */
       if (problem)
-	problem = krb5_get_in_tkt_with_password(ssh_context,
-						krb5_options, 0,
-						NULL, 0,
-						password, ccache,
-						&my_creds, 0);
+        problem = krb5_get_in_tkt_with_password(ssh_context,
+                                                krb5_options, 0,
+                                                NULL, 0,
+                                                password, ccache,
+                                                &my_creds, 0);
       krb5_free_principal(ssh_context, server);
       server = 0;
       if (problem)
-	goto errout;
+        goto errout;
       else
-	{
-	  /* Verify tgt just obtained */
-	  results = verify_krb_v5_tgt(ssh_context, ccache, &problem);
-	  
-	  if (results  >= 0)
-	    {
-	      /* If results is 0 then put out a log message that
-		 the TGT was not verified, pass this back to the
-		 user as well */
-	      if (results == 0)
-		{
-		  log_msg("TGT for user %s was not verified.", server_user);
-		  packet_send_debug("Kerberos TGT could not be verified.");
-		}
-	      
+        {
+          /* Verify tgt just obtained */
+          results = verify_krb_v5_tgt(ssh_context, ccache, &problem);
+          
+          if (results  >= 0)
+            {
+              /* If results is 0 then put out a log message that
+                 the TGT was not verified, pass this back to the
+                 user as well */
+              if (results == 0)
+                {
+                  log_msg("TGT for user %.100s was not verified.",
+                          server_user);
+                  packet_send_debug("Kerberos TGT could not be verified.");
+                }
+              
             /* get_name pulls out just the name not the
                type */
-	      strcpy(ccname + 5, krb5_cc_get_name(ssh_context, ccache));
-	      (void) chown(ccname + 5, pw->pw_uid, pw->pw_gid);
-	      
-	      /* If tgt was passed unlink file */
-	      if (ticket)
-		{
-		  if (strcmp(ticket,"none"))
-		    /* ticket -> FILE:path */
-		    unlink(ticket + 5);
-		  else
+              strcpy(ccname + 5, krb5_cc_get_name(ssh_context, ccache));
+              (void) chown(ccname + 5, pw->pw_uid, pw->pw_gid);
+              
+              /* If tgt was passed unlink file */
+              if (ticket)
+                {
+                  if (strcmp(ticket,"none"))
+                    /* ticket -> FILE:path */
+                    unlink(ticket + 5);
+                  else
                     ticket = NULL;
-		}
-	      
-	      ticket = xmalloc(strlen(ccname) + 1);
-	      (void) sprintf(ticket, "%s", ccname);
-	      
-	      /* We do not need this so free them up */
-	      xfree(saved_pw_name);
-	      xfree(saved_pw_passwd);
-	      
-	      return 1;
-	    }
-	  if (problem == KRB5_KT_NOTFOUND)
-	    {
-	      /* We have potential KDC spoofing here - log it */
-	      log_msg("WARNING: Verification of TGT indicates potential KDC spoofing: user %s address %s", server_user, get_remote_ipaddr());
-	      packet_send_debug("Verification of TGT indicates potential KDC spoofing.");
-	    }
-	}
+                }
+              
+              ticket = xmalloc(strlen(ccname) + 1);
+              (void) snprintf(ticket, strlen(ccname) + 1, "%s", ccname);
+              
+              /* We do not need this so free them up */
+              xfree(saved_pw_name);
+              xfree(saved_pw_passwd);
+              
+              return 1;
+            }
+          if (problem == KRB5_KT_NOTFOUND)
+            {
+              /* We have potential KDC spoofing here - log it */
+              log_msg("WARNING: Verification of TGT indicates potential KDC spoofing: user %.100s address %.100s", server_user, get_remote_ipaddr());
+              packet_send_debug("Verification of TGT indicates potential KDC spoofing.");
+            }
+        }
     errout:
       krb5_cc_destroy (ssh_context, ccache);
     errout2:
       if (problem)
-	{
-	  log_msg("Password authentication of user %s using Kerberos failed: %s",
-		  server_user, error_message(problem));
-	  if (server) 
-	    krb5_free_principal(ssh_context, server);
-	  if (!options.kerberos_or_local_passwd )
-	    {
-	      /* We do not need this so free them up */
-	      xfree(saved_pw_name);
-	      xfree(saved_pw_passwd);
-	      return 0;
-	    }
-	}
+        {
+          log_msg("Password authentication of user %.100s using Kerberos failed: %.100s",
+                  server_user, error_message(problem));
+          if (server) 
+            krb5_free_principal(ssh_context, server);
+          if (!options.kerberos_or_local_passwd )
+            {
+              /* We do not need this so free them up */
+              xfree(saved_pw_name);
+              xfree(saved_pw_passwd);
+              return 0;
+            }
+        }
 #endif /* KRB5 */
     }
 #endif /* KERBEROS */
@@ -657,45 +677,45 @@ int auth_password(const char *server_user, const char *password)
     
     if (securid_users)
       {
-	while (fgets(su_user, sizeof(su_user), securid_users))
-	  {
-	    if (c = strchr(su_user, '\n')) 
-	      *c = '\0';
-	    if (strcmp(su_user, server_user) == 0) 
-	      { 
-		found = 1; 
-		break; 
-	      }
-	  }
+        while (fgets(su_user, sizeof(su_user), securid_users))
+          {
+            if (c = strchr(su_user, '\n')) 
+              *c = '\0';
+            if (strcmp(su_user, server_user) == 0) 
+              { 
+                found = 1; 
+                break; 
+              }
+          }
       }
     fclose(securid_users);
 
     if (found)
       {
-	/* The user has a SecurID card. */
-	struct SD_CLIENT sd_dat, *sd;
-	log_msg("SecurID authentication for %.100s required.", server_user);
+        /* The user has a SecurID card. */
+        struct SD_CLIENT sd_dat, *sd;
+        log_msg("SecurID authentication for %.100s required.", server_user);
 
-	/*
-	 * if no pass code has been supplied, fail immediately: passing
-	 * a null pass code to sd_check causes a core dump
-	 */
-	if (*password == '\0') 
-	  {
-	    log_msg("No pass code given, authentication rejected.");
-	    return 0;
-	  }
+        /*
+         * if no pass code has been supplied, fail immediately: passing
+         * a null pass code to sd_check causes a core dump
+         */
+        if (*password == '\0') 
+          {
+            log_msg("No pass code given, authentication rejected.");
+            return 0;
+          }
 
-	sd = &sd_dat;
-	if (!securid_initialized)
-	  {
-	    memset(&sd_dat, 0, sizeof(sd_dat));   /* clear struct */
-	    creadcfg();		/*  accesses sdconf.rec  */
-	    if (sd_init(sd)) 
-	      packet_disconnect("Cannot contact securid server.");
-	    securid_initialized = 1;
-	  }
-	return sd_check(password, server_user, sd) == ACM_OK;
+        sd = &sd_dat;
+        if (!securid_initialized)
+          {
+            memset(&sd_dat, 0, sizeof(sd_dat));   /* clear struct */
+            creadcfg();         /*  accesses sdconf.rec  */
+            if (sd_init(sd)) 
+              packet_disconnect("Cannot contact securid server.");
+            securid_initialized = 1;
+          }
+        return sd_check(password, server_user, sd) == ACM_OK;
       }
   }
   /* If the user has no SecurID card specified, we fall to normal 
@@ -712,24 +732,6 @@ int auth_password(const char *server_user, const char *password)
   seteuid(UID_ROOT);                /* just let it fail if ran by user */
 #endif /* SECURE_RPC */
 
-#ifdef HAVE_OSF1_C2_SECURITY
-  if (osf1c2_getprpwent(correct_passwd, saved_pw_name,
-			sizeof(correct_passwd)))
-    {
-      if (options.forced_passwd_change)
-	{
-	  extern char *forced_command;
-	  forced_command = xmalloc(sizeof(PASSWD_PATH) +
-				   strlen(server_user) + 1);
-	  sprintf(forced_command, "%s %s", PASSWD_PATH, server_user);
-	  packet_send_debug("Password expired, forcing it to be changed.");
-	}
-      else
-	{
-	  packet_send_debug("\n\tYour password has expired.");
-	}
-    }
-#else /* HAVE_OSF1_C2_SECURITY */
   /* If we have shadow passwords, lookup the real encrypted password from
      the shadow file, and replace the saved encrypted password with the
      real encrypted password. */
@@ -742,19 +744,19 @@ int auth_password(const char *server_user, const char *password)
         strncpy(correct_passwd, pr->ufld.fd_encrypt, sizeof(correct_passwd));
         endprpwent();
         if ( (!pr->uflg.fg_nullpw || !pr->ufld.fd_nullpw)
-	     && !pr->uflg.fg_pw_admin_num
-	     && strcmp(correct_passwd,"")==0 )
+             && !pr->uflg.fg_pw_admin_num
+             && strcmp(correct_passwd,"")==0 )
           {
-	    debug("User %.100s not permitted to login with null passwd",
+            debug("User %.100s not permitted to login with null passwd",
                   saved_pw_name);
-	    packet_send_debug("\n\tNot permitted null passwd.");
-	    return 0;
+            packet_send_debug("\n\tNot permitted null passwd.");
+            return 0;
           }
       }
     else
       {       
-	endprpwent();
-	return 0;
+        endprpwent();
+        /* Continue with other alternatives (or NIS will cease to work). */
       }
   }
 #else /* defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH) */
@@ -763,12 +765,12 @@ int auth_password(const char *server_user, const char *password)
     struct spwd *sp = getspnam(saved_pw_name);
 #if defined(SECURE_RPC) && defined(NIS_PLUS)
     if (!geteuid() && pw->pw_uid && /* do we have guts? */
-	(!sp || !sp->sp_pwdp || !strcmp(sp->sp_pwdp,"*NP*")))
+        (!sp || !sp->sp_pwdp || !strcmp(sp->sp_pwdp,"*NP*")))
       if (seteuid(pw->pw_uid) == UID_ROOT)
-	{
-	  sp = getspnam(saved_pw_name); /* retry as user */   
-	  seteuid(UID_ROOT);
-	}
+        {
+          sp = getspnam(saved_pw_name); /* retry as user */   
+          seteuid(UID_ROOT);
+        }
 #endif /* SECURE_RPC && NIS_PLUS */
      if (sp)
       strncpy(correct_passwd, sp->sp_pwdp, sizeof(correct_passwd));
@@ -791,60 +793,98 @@ int auth_password(const char *server_user, const char *password)
     f = fopen("/etc/security/passwd", "r");
     if (f)
       {
-	sprintf(looking_for_user, "%.190s:", server_user);
-	while (fgets(line, sizeof(line), f))
-	  {
-	    if (strchr(line, '\n'))
-	      *strchr(line, '\n') = 0;
-	    if (strcmp(line, looking_for_user) == 0)
-	      found_user = 1;
-	    else
-	      if (line[0] != '\t' && line[0] != ' ')
-		found_user = 0;
-	      else
-		if (found_user)
-		  {
-		    for (cp = line; *cp == ' ' || *cp == '\t'; cp++)
-		      ;
-		    if (strncmp(cp, "password = ", strlen("password = ")) == 0)
-		      {
-			strncpy(correct_passwd, cp + strlen("password = "), 
-				sizeof(correct_passwd));
-			correct_passwd[sizeof(correct_passwd) - 1] = 0;
-			break;
-		      }
-		  }
-	  }
-	fclose(f);
+        snprintf(looking_for_user, sizeof(looking_for_user),
+                 "%.190s:", server_user);
+        while (fgets(line, sizeof(line), f))
+          {
+            if (strchr(line, '\n'))
+              *strchr(line, '\n') = 0;
+            if (strcmp(line, looking_for_user) == 0)
+              found_user = 1;
+            else
+              if (line[0] != '\t' && line[0] != ' ')
+                found_user = 0;
+              else
+                if (found_user)
+                  {
+                    for (cp = line; *cp == ' ' || *cp == '\t'; cp++)
+                      ;
+                    if (strncmp(cp, "password = ", strlen("password = ")) == 0)
+                      {
+                        strncpy(correct_passwd, cp + strlen("password = "), 
+                                sizeof(correct_passwd));
+                        correct_passwd[sizeof(correct_passwd) - 1] = 0;
+                        break;
+                      }
+                  }
+          }
+        fclose(f);
       }
   }
 #endif /* HAVE_ETC_SECURITY_PASSWD */
 #endif /* HAVE_ETC_SECURITY_PASSWD_ADJUNCT */
 #endif /* HAVE_ETC_SHADOW */
 #endif /* HAVE_SCO_ETC_SHADOW */
-#endif /* HAVE_OSF1_C2_SECURITY */
 
   /* Check for users with no password. */
+#ifdef HAVE_SIA
+  if (strcmp(password, "") == 0 && sia_no_password(server_user))
+#else /* HAVE_SIA */
   if (strcmp(password, "") == 0 && strcmp(correct_passwd, "") == 0)
+#endif /* HAVE_SIA */
     {
       if (options.forced_empty_passwd_change)
-	{
-	  extern char *forced_command;
-	  forced_command = xmalloc(sizeof(PASSWD_PATH) +
-				   strlen(server_user) + 1);
-	  sprintf(forced_command, "%s %s", PASSWD_PATH, server_user);
+        {
+          extern char *forced_command;
+          forced_command = xmalloc(sizeof(PASSWD_PATH) +
+                                   strlen(server_user) + 2);
+          snprintf(forced_command, 
+                   sizeof(PASSWD_PATH) + strlen(server_user) + 2,
+                   "%.100s %.100s", PASSWD_PATH, server_user);
           packet_send_debug("Password if forced to be set at first login.");
-	}
+        }
       else
-	{
-	  packet_send_debug("Login permitted without a password because the account has no password.");
-	}
+        {
+          packet_send_debug("Login permitted without a password because the account has no password.");
+        }
       return 1; /* The user has no password and an empty password was tried. */
     }
 
   xfree(saved_pw_name);
   xfree(saved_pw_passwd);
 
+#ifdef HAVE_SIA
+  {
+    int argc;
+    char **argv;
+    const char *hostname = get_canonical_hostname();
+
+    /* Passing a collection routine to my_sia_validate_user() here would
+       be useless and could be harmful.
+
+       It would be useless because at this point, stdin/stdout/stderr
+       are all redirected to /dev/null.
+
+       It would be harmful if the collection routine tried to print
+       anything to stdout, say, a warning that the password is incorrect.
+       At this point, nothing has been printed to stdout yet and it's
+       redirected to /dev/null.  Printing to stdout now would initialize
+       it as fully buffered (not line buffered) since /dev/null isn't a
+       tty.  That would cause problems later on in the child, which gets
+       a copy of the stdout structure.  When the child printed /etc/motd
+       and the mail notice, the user wouldn't see them because they'd
+       be stuck in the stdout buffer. */
+
+    get_sia_args(&argc, &argv);
+    if (my_sia_validate_user(NULL, argc, argv,
+                             (char *)hostname, (char *)server_user,
+                             NULL, 0, NULL, (char *)password) == SIASUCCESS)
+      return 1;
+    else
+      return 0;
+  }
+#endif /* HAVE_SIA */
+
 #ifdef HAVE_ULTRIX_SHADOW_PASSWORDS
   {
     /* Note: we are assuming that pw from above is still valid. */
@@ -852,32 +892,26 @@ int auth_password(const char *server_user, const char *password)
     svp = getsvc();
     if (svp == NULL)
       {
-	error("getsvc() failed in ultrix code in auth_passwd");
-	return 0;
+        error("getsvc() failed in ultrix code in auth_passwd");
+        return 0;
       }
     if ((svp->svcauth.seclevel == SEC_UPGRADE &&
-	 strcmp(pw->pw_passwd, "*") == 0) ||
-	svp->svcauth.seclevel == SEC_ENHANCED)
+         strcmp(pw->pw_passwd, "*") == 0) ||
+        svp->svcauth.seclevel == SEC_ENHANCED)
       return authenticate_user(pw, password, "/dev/ttypXX") >= 0;
   }
 #endif /* HAVE_ULTRIX_SHADOW_PASSWORDS */
 
   /* Encrypt the candidate password using the proper salt. */
-#ifdef HAVE_OSF1_C2_SECURITY
-  encrypted_password = (char *)osf1c2crypt(password,
-                                   (correct_passwd[0] && correct_passwd[1]) ?
-                                   correct_passwd : "xx");
-#else /* HAVE_OSF1_C2_SECURITY */
 #if defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH)
   encrypted_password = bigcrypt(password, 
-			     (correct_passwd[0] && correct_passwd[1]) ?
-			     correct_passwd : "xx");
+                             (correct_passwd[0] && correct_passwd[1]) ?
+                             correct_passwd : "xx");
 #else /* defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH) */
   encrypted_password = crypt(password, 
-			     (correct_passwd[0] && correct_passwd[1]) ?
-			     correct_passwd : "xx");
+                             (correct_passwd[0] && correct_passwd[1]) ?
+                             correct_passwd : "xx");
 #endif /* HAVE_SCO_ETC_SHADOW */
-#endif /* HAVE_OSF1_C2_SECURITY */
 
   /* Authentication is accepted if the encrypted passwords are identical. */
   return strcmp(encrypted_password, correct_passwd) == 0;
