@@ -14,31 +14,36 @@ Functions for reading passphrases and passwords.
 */
 
 /*
- * $Id: readpass.c,v 1.6 1997/05/13 22:30:18 kivinen Exp $
+ * $Id: readpass.c,v 1.7 1999/02/21 19:52:33 ylo Exp $
  * $Log: readpass.c,v $
- * Revision 1.6  1997/05/13  22:30:18  kivinen
- * 	Added some casts.
+ * Revision 1.7  1999/02/21 19:52:33  ylo
+ * 	Intermediate commit of ssh1.2.27 stuff.
+ * 	Main change is sprintf -> snprintf; however, there are also
+ * 	many other changes.
+ *
+ * Revision 1.6  1997/05/13 22:30:18  kivinen
+ *      Added some casts.
  *
  * Revision 1.5  1997/04/17 04:01:52  kivinen
- * 	Added read_confirmation function.
+ *      Added read_confirmation function.
  *
  * Revision 1.4  1997/04/05 21:49:28  kivinen
- * 	Fixed the '-quoting from \' to '\''.
+ *      Fixed the '-quoting from \' to '\''.
  *
  * Revision 1.3  1997/03/26 07:15:23  kivinen
- * 	Fixed prompt quoting so ' will be quoted only if in command
- * 	line.
+ *      Fixed prompt quoting so ' will be quoted only if in command
+ *      line.
  *
  * Revision 1.2  1997/03/19 17:36:18  kivinen
- * 	Quote all unprintable characters in password prompt. Also
- * 	quote all '-characters.
+ *      Quote all unprintable characters in password prompt. Also
+ *      quote all '-characters.
  *
  * Revision 1.1.1.1  1996/02/18 21:38:12  ylo
- * 	Imported ssh-1.2.13.
+ *      Imported ssh-1.2.13.
  *
  * Revision 1.2  1995/07/13  01:31:04  ylo
- * 	Removed "Last modified" header.
- * 	Added cvs log.
+ *      Removed "Last modified" header.
+ *      Added cvs log.
  *
  * $Endlog$
  */
@@ -101,86 +106,87 @@ char *read_passphrase(uid_t uid, const char *prompt, int from_stdin)
   else
     {
       /* Read the passphrase from /dev/tty to make it possible to ask it even 
-	 when stdin has been redirected. */
+         when stdin has been redirected. */
       f = fopen("/dev/tty", "r");
       if (!f)
-	{
-	  if (getenv("DISPLAY"))
-	    {
-	      char command[512];
-	      
-	      fprintf(stderr,
-		      "Executing ssh-askpass to query the password...\n");
-	      fflush(stdout);
-	      fflush(stderr);
-	      for(p = (unsigned const char *) prompt, i = 0;
-		  i < sizeof(quoted_prompt) - 5 && *p;
-		  i++, p++)
-		{
-		  if (*p == '\'')
-		    {
-		      quoted_prompt[i++] = '\'';
-		      quoted_prompt[i++] = '\\';
-		      quoted_prompt[i++] = '\'';
-		      quoted_prompt[i] = '\'';
-		    }
-		  else if (isprint(*p) || isspace(*p))
-		    quoted_prompt[i] = *p;
-		  else if (iscntrl(*p))
-		    {
-		      quoted_prompt[i++] = '^';
-		      if (*p < ' ')
-			quoted_prompt[i] = *p + '@';
-		      else
-			quoted_prompt[i] = '?';
-		    }
-		  else if (*p > 128)
-		    quoted_prompt[i] = *p;
-		}
-	      quoted_prompt[i] = '\0';
+        {
+          if (getenv("DISPLAY"))
+            {
+              char command[512];
+              
+              fprintf(stderr,
+                      "Executing ssh-askpass to query the password...\n");
+              fflush(stdout);
+              fflush(stderr);
+              for(p = (unsigned const char *) prompt, i = 0;
+                  i < sizeof(quoted_prompt) - 5 && *p;
+                  i++, p++)
+                {
+                  if (*p == '\'')
+                    {
+                      quoted_prompt[i++] = '\'';
+                      quoted_prompt[i++] = '\\';
+                      quoted_prompt[i++] = '\'';
+                      quoted_prompt[i] = '\'';
+                    }
+                  else if (isprint(*p) || isspace(*p))
+                    quoted_prompt[i] = *p;
+                  else if (iscntrl(*p))
+                    {
+                      quoted_prompt[i++] = '^';
+                      if (*p < ' ')
+                        quoted_prompt[i] = *p + '@';
+                      else
+                        quoted_prompt[i] = '?';
+                    }
+                  else if (*p > 128)
+                    quoted_prompt[i] = *p;
+                }
+              quoted_prompt[i] = '\0';
   
-	      sprintf(command, "ssh-askpass '%.400s'", quoted_prompt);
-	      
-	      uf = userfile_popen(uid, command, "r");
-	      if (uf == NULL)
-		{
-		  fprintf(stderr, "Could not query passphrase: '%.200s' failed.\n",
-			  command);
-		  exit(1);
-		}
-	      if (!userfile_gets(buf, sizeof(buf), uf))
-		{
-		  userfile_pclose(uf);
-		  fprintf(stderr, "No passphrase supplied.  Exiting.\n");
-		  exit(1);
-		}
-	      userfile_pclose(uf);
-	      if (strchr(buf, '\n'))
-		*strchr(buf, '\n') = 0;
-	      return xstrdup(buf);
-	    }
+              snprintf(command, sizeof(command),
+                       "ssh-askpass '%.400s'", quoted_prompt);
+              
+              uf = userfile_popen(uid, command, "r");
+              if (uf == NULL)
+                {
+                  fprintf(stderr, "Could not query passphrase: '%.200s' failed.\n",
+                          command);
+                  exit(1);
+                }
+              if (!userfile_gets(buf, sizeof(buf), uf))
+                {
+                  userfile_pclose(uf);
+                  fprintf(stderr, "No passphrase supplied.  Exiting.\n");
+                  exit(1);
+                }
+              userfile_pclose(uf);
+              if (strchr(buf, '\n'))
+                *strchr(buf, '\n') = 0;
+              return xstrdup(buf);
+            }
 
-	  /* No controlling terminal and no DISPLAY.  Nowhere to read. */
-	  fprintf(stderr, "You have no controlling tty and no DISPLAY.  Cannot read passphrase.\n");
-	  exit(1);
-	}
+          /* No controlling terminal and no DISPLAY.  Nowhere to read. */
+          fprintf(stderr, "You have no controlling tty and no DISPLAY.  Cannot read passphrase.\n");
+          exit(1);
+        }
     }
 
   for(p = (unsigned const char *) prompt, i = 0;
       i < sizeof(quoted_prompt) - 4 && *p; i++, p++)
     {
       if (isprint(*p) || isspace(*p))
-	quoted_prompt[i] = *p;
+        quoted_prompt[i] = *p;
       else if (iscntrl(*p))
-	{
-	  quoted_prompt[i++] = '^';
-	  if (*p < ' ')
-	    quoted_prompt[i] = *p + '@';
-	  else
-	    quoted_prompt[i] = '?';
-	}
+        {
+          quoted_prompt[i++] = '^';
+          if (*p < ' ')
+            quoted_prompt[i] = *p + '@';
+          else
+            quoted_prompt[i] = '?';
+        }
       else if (*p > 128)
-	quoted_prompt[i] = *p;
+        quoted_prompt[i] = *p;
     }
   quoted_prompt[i] = '\0';
   
@@ -227,7 +233,7 @@ char *read_passphrase(uid_t uid, const char *prompt, int from_stdin)
       fprintf(stderr, "\n");
       /* Close the file. */
       if (f != stdin)
-	fclose(f);
+        fclose(f);
       exit(1);
     }
   /* Restore terminal modes. */
@@ -268,13 +274,13 @@ void read_confirmation(const char *prompt)
   else
     {
       /* Read the passphrase from /dev/tty to make it possible to ask it even 
-	 when stdin has been redirected. */
+         when stdin has been redirected. */
       f = fopen("/dev/tty", "r");
       if (!f)
-	{
-	  fprintf(stderr, "You have no controlling tty.  Cannot read confirmation.\n");
-	  exit(1);
-	}
+        {
+          fprintf(stderr, "You have no controlling tty.  Cannot read confirmation.\n");
+          exit(1);
+        }
     }
 
   /* Read the passphrase from the terminal. */
@@ -286,29 +292,29 @@ void read_confirmation(const char *prompt)
       fflush(stderr);
       /* Read line */
       if (fgets(buf, sizeof(buf), f) == NULL)
-	{
-	  /* Got EOF.  Just exit. */
-	  /* Print a newline (the prompt probably didn\'t have one). */
-	  fprintf(stderr, "\n");
-	  fprintf(stderr, "Aborted by user");
-	  /* Close the file. */
-	  if (f != stdin)
-	    fclose(f);
-	  exit(1);
-	}
+        {
+          /* Got EOF.  Just exit. */
+          /* Print a newline (the prompt probably didn\'t have one). */
+          fprintf(stderr, "\n");
+          fprintf(stderr, "Aborted by user");
+          /* Close the file. */
+          if (f != stdin)
+            fclose(f);
+          exit(1);
+        }
       p = buf + strlen(buf) - 1;
       while (p > buf && isspace(*p))
-	*p-- = '\0';
+        *p-- = '\0';
       p = buf;
       while (*p && isspace(*p))
-	p++;
+        p++;
       if (strcmp(p, "no") == 0)
-	{
-	  /* Close the file. */
-	  if (f != stdin)
-	    fclose(f);
-	  exit(1);
-	}
+        {
+          /* Close the file. */
+          if (f != stdin)
+            fclose(f);
+          exit(1);
+        }
     } while (strcmp(p, "yes") != 0);
   /* Close the file. */
   if (f != stdin)
